# pylint: disable=too-many-lines
# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import datetime
from typing import Any, Callable, Dict, List, Optional, TypeVar

from azure.core.exceptions import ClientAuthenticationError, HttpResponseError, ResourceExistsError, ResourceNotFoundError, map_error
from azure.core.pipeline import PipelineResponse
from azure.core.pipeline.transport import AsyncHttpResponse
from azure.core.rest import HttpRequest
from azure.core.tracing.decorator_async import distributed_trace_async

from ... import models as _models
from ..._vendor import _convert_request
from ...operations._machine_alarms_operations import build_get_machine_alarm_details_request, build_list_alarm_severity_occurrences_request, build_list_alarms_per_machine_request, build_list_machine_alarms_request
T = TypeVar('T')
ClsType = Optional[Callable[[PipelineResponse[HttpRequest, AsyncHttpResponse], T, Dict[str, Any]], Any]]

class MachineAlarmsOperations:
    """
    .. warning::
        **DO NOT** instantiate this class directly.

        Instead, you should access the following operations through
        :class:`~ignos.api.client.aio.IgnosPortal`'s
        :attr:`machine_alarms` attribute.
    """

    models = _models

    def __init__(self, *args, **kwargs) -> None:
        input_args = list(args)
        self._client = input_args.pop(0) if input_args else kwargs.pop("client")
        self._config = input_args.pop(0) if input_args else kwargs.pop("config")
        self._serialize = input_args.pop(0) if input_args else kwargs.pop("serializer")
        self._deserialize = input_args.pop(0) if input_args else kwargs.pop("deserializer")


    @distributed_trace_async
    async def list_machine_alarms(
        self,
        asset_id: Optional[int] = None,
        start_time: Optional[datetime.datetime] = None,
        end_time: Optional[datetime.datetime] = None,
        **kwargs: Any
    ) -> List[_models.MachineAlarmDto]:
        """list_machine_alarms.

        :param asset_id:  Default value is None.
        :type asset_id: long
        :param start_time:  Default value is None.
        :type start_time: ~datetime.datetime
        :param end_time:  Default value is None.
        :type end_time: ~datetime.datetime
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: list of MachineAlarmDto, or the result of cls(response)
        :rtype: list[~ignos.api.client.models.MachineAlarmDto]
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop('cls', None)  # type: ClsType[List[_models.MachineAlarmDto]]

        
        request = build_list_machine_alarms_request(
            asset_id=asset_id,
            start_time=start_time,
            end_time=end_time,
            template_url=self.list_machine_alarms.metadata['url'],
            headers=_headers,
            params=_params,
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = await self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request,
            stream=False,
            **kwargs
        )
        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize('[MachineAlarmDto]', pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    list_machine_alarms.metadata = {'url': "/machinealarms"}  # type: ignore


    @distributed_trace_async
    async def list_alarms_per_machine(
        self,
        start_time: Optional[datetime.datetime] = None,
        end_time: Optional[datetime.datetime] = None,
        asset_id: Optional[int] = None,
        **kwargs: Any
    ) -> List[_models.MachineAlarmSummaryDto]:
        """list_alarms_per_machine.

        :param start_time:  Default value is None.
        :type start_time: ~datetime.datetime
        :param end_time:  Default value is None.
        :type end_time: ~datetime.datetime
        :param asset_id:  Default value is None.
        :type asset_id: long
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: list of MachineAlarmSummaryDto, or the result of cls(response)
        :rtype: list[~ignos.api.client.models.MachineAlarmSummaryDto]
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop('cls', None)  # type: ClsType[List[_models.MachineAlarmSummaryDto]]

        
        request = build_list_alarms_per_machine_request(
            start_time=start_time,
            end_time=end_time,
            asset_id=asset_id,
            template_url=self.list_alarms_per_machine.metadata['url'],
            headers=_headers,
            params=_params,
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = await self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request,
            stream=False,
            **kwargs
        )
        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize('[MachineAlarmSummaryDto]', pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    list_alarms_per_machine.metadata = {'url': "/machinealarms/alarmspermachine"}  # type: ignore


    @distributed_trace_async
    async def list_alarm_severity_occurrences(
        self,
        start_time: Optional[datetime.datetime] = None,
        end_time: Optional[datetime.datetime] = None,
        asset_id: Optional[int] = None,
        **kwargs: Any
    ) -> List[_models.MachineAlarmSeverityOccurenceDto]:
        """list_alarm_severity_occurrences.

        :param start_time:  Default value is None.
        :type start_time: ~datetime.datetime
        :param end_time:  Default value is None.
        :type end_time: ~datetime.datetime
        :param asset_id:  Default value is None.
        :type asset_id: long
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: list of MachineAlarmSeverityOccurenceDto, or the result of cls(response)
        :rtype: list[~ignos.api.client.models.MachineAlarmSeverityOccurenceDto]
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop('cls', None)  # type: ClsType[List[_models.MachineAlarmSeverityOccurenceDto]]

        
        request = build_list_alarm_severity_occurrences_request(
            start_time=start_time,
            end_time=end_time,
            asset_id=asset_id,
            template_url=self.list_alarm_severity_occurrences.metadata['url'],
            headers=_headers,
            params=_params,
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = await self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request,
            stream=False,
            **kwargs
        )
        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize('[MachineAlarmSeverityOccurenceDto]', pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    list_alarm_severity_occurrences.metadata = {'url': "/machinealarms/alarmtypeseverities"}  # type: ignore


    @distributed_trace_async
    async def get_machine_alarm_details(
        self,
        id: int,
        **kwargs: Any
    ) -> _models.MachineAlarmDetailsDto:
        """get_machine_alarm_details.

        :param id:
        :type id: long
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: MachineAlarmDetailsDto, or the result of cls(response)
        :rtype: ~ignos.api.client.models.MachineAlarmDetailsDto
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop('cls', None)  # type: ClsType[_models.MachineAlarmDetailsDto]

        
        request = build_get_machine_alarm_details_request(
            id=id,
            template_url=self.get_machine_alarm_details.metadata['url'],
            headers=_headers,
            params=_params,
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = await self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request,
            stream=False,
            **kwargs
        )
        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize('MachineAlarmDetailsDto', pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    get_machine_alarm_details.metadata = {'url': "/machinealarms/alarms/{id}"}  # type: ignore

