# pylint: disable=too-many-lines
# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import datetime
from typing import TYPE_CHECKING

from msrest import Serializer

from azure.core.exceptions import ClientAuthenticationError, HttpResponseError, ResourceExistsError, ResourceNotFoundError, map_error
from azure.core.pipeline import PipelineResponse
from azure.core.pipeline.transport import HttpResponse
from azure.core.rest import HttpRequest
from azure.core.tracing.decorator import distributed_trace
from azure.core.utils import case_insensitive_dict

from .. import models as _models
from .._vendor import _convert_request, _format_url_section

if TYPE_CHECKING:
    # pylint: disable=unused-import,ungrouped-imports
    from typing import Any, Callable, Dict, List, Optional, TypeVar
    T = TypeVar('T')
    ClsType = Optional[Callable[[PipelineResponse[HttpRequest, HttpResponse], T, Dict[str, Any]], Any]]

_SERIALIZER = Serializer()
_SERIALIZER.client_side_validation = False
# fmt: off

def build_get_machine_utilizations_request(
    **kwargs  # type: Any
):
    # type: (...) -> HttpRequest
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    asset_id = kwargs.pop('asset_id', _params.pop('assetId', None))  # type: Optional[int]
    favorites = kwargs.pop('favorites', _params.pop('favorites', False))  # type: Optional[bool]
    accept = _headers.pop('Accept', "application/json")

    # Construct URL
    _url = kwargs.pop("template_url", "/machineutilization")

    # Construct parameters
    if asset_id is not None:
        _params['assetId'] = _SERIALIZER.query("asset_id", asset_id, 'long')
    if favorites is not None:
        _params['favorites'] = _SERIALIZER.query("favorites", favorites, 'bool')

    # Construct headers
    _headers['Accept'] = _SERIALIZER.header("accept", accept, 'str')

    return HttpRequest(
        method="GET",
        url=_url,
        params=_params,
        headers=_headers,
        **kwargs
    )


def build_get_utilization_data_point_for_machine_request(
    id,  # type: int
    **kwargs  # type: Any
):
    # type: (...) -> HttpRequest
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    start_time = kwargs.pop('start_time', _params.pop('startTime', None))  # type: Optional[datetime.datetime]
    accept = _headers.pop('Accept', "application/json")

    # Construct URL
    _url = kwargs.pop("template_url", "/machineutilization/{id}/utilizationdatapoints")
    path_format_arguments = {
        "id": _SERIALIZER.url("id", id, 'long'),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    # Construct parameters
    if start_time is not None:
        _params['startTime'] = _SERIALIZER.query("start_time", start_time, 'iso-8601')

    # Construct headers
    _headers['Accept'] = _SERIALIZER.header("accept", accept, 'str')

    return HttpRequest(
        method="GET",
        url=_url,
        params=_params,
        headers=_headers,
        **kwargs
    )


def build_get_utilization_details_for_machine_request(
    id,  # type: int
    **kwargs  # type: Any
):
    # type: (...) -> HttpRequest
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    start_time = kwargs.pop('start_time', _params.pop('startTime', None))  # type: Optional[datetime.datetime]
    accept = _headers.pop('Accept', "application/json")

    # Construct URL
    _url = kwargs.pop("template_url", "/machineutilization/{id}/utilization")
    path_format_arguments = {
        "id": _SERIALIZER.url("id", id, 'long'),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    # Construct parameters
    if start_time is not None:
        _params['startTime'] = _SERIALIZER.query("start_time", start_time, 'iso-8601')

    # Construct headers
    _headers['Accept'] = _SERIALIZER.header("accept", accept, 'str')

    return HttpRequest(
        method="GET",
        url=_url,
        params=_params,
        headers=_headers,
        **kwargs
    )

# fmt: on
class MachineUtilizationOperations(object):
    """
    .. warning::
        **DO NOT** instantiate this class directly.

        Instead, you should access the following operations through
        :class:`~ignos.api.client.IgnosPortal`'s
        :attr:`machine_utilization` attribute.
    """

    models = _models

    def __init__(self, *args, **kwargs):
        input_args = list(args)
        self._client = input_args.pop(0) if input_args else kwargs.pop("client")
        self._config = input_args.pop(0) if input_args else kwargs.pop("config")
        self._serialize = input_args.pop(0) if input_args else kwargs.pop("serializer")
        self._deserialize = input_args.pop(0) if input_args else kwargs.pop("deserializer")


    @distributed_trace
    def get_machine_utilizations(
        self,
        asset_id=None,  # type: Optional[int]
        favorites=False,  # type: Optional[bool]
        **kwargs  # type: Any
    ):
        # type: (...) -> _models.UtilizationDto
        """get_machine_utilizations.

        :param asset_id:  Default value is None.
        :type asset_id: long
        :param favorites:  Default value is False.
        :type favorites: bool
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: UtilizationDto, or the result of cls(response)
        :rtype: ~ignos.api.client.models.UtilizationDto
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop('cls', None)  # type: ClsType[_models.UtilizationDto]

        
        request = build_get_machine_utilizations_request(
            asset_id=asset_id,
            favorites=favorites,
            template_url=self.get_machine_utilizations.metadata['url'],
            headers=_headers,
            params=_params,
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request,
            stream=False,
            **kwargs
        )
        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize('UtilizationDto', pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    get_machine_utilizations.metadata = {'url': "/machineutilization"}  # type: ignore


    @distributed_trace
    def get_utilization_data_point_for_machine(
        self,
        id,  # type: int
        start_time=None,  # type: Optional[datetime.datetime]
        **kwargs  # type: Any
    ):
        # type: (...) -> List[_models.NumericDataPointDto]
        """get_utilization_data_point_for_machine.

        :param id:
        :type id: long
        :param start_time:  Default value is None.
        :type start_time: ~datetime.datetime
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: list of NumericDataPointDto, or the result of cls(response)
        :rtype: list[~ignos.api.client.models.NumericDataPointDto]
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop('cls', None)  # type: ClsType[List[_models.NumericDataPointDto]]

        
        request = build_get_utilization_data_point_for_machine_request(
            id=id,
            start_time=start_time,
            template_url=self.get_utilization_data_point_for_machine.metadata['url'],
            headers=_headers,
            params=_params,
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request,
            stream=False,
            **kwargs
        )
        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize('[NumericDataPointDto]', pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    get_utilization_data_point_for_machine.metadata = {'url': "/machineutilization/{id}/utilizationdatapoints"}  # type: ignore


    @distributed_trace
    def get_utilization_details_for_machine(
        self,
        id,  # type: int
        start_time=None,  # type: Optional[datetime.datetime]
        **kwargs  # type: Any
    ):
        # type: (...) -> _models.UtilizationDetailsDto
        """get_utilization_details_for_machine.

        :param id:
        :type id: long
        :param start_time:  Default value is None.
        :type start_time: ~datetime.datetime
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: UtilizationDetailsDto, or the result of cls(response)
        :rtype: ~ignos.api.client.models.UtilizationDetailsDto
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls = kwargs.pop('cls', None)  # type: ClsType[_models.UtilizationDetailsDto]

        
        request = build_get_utilization_details_for_machine_request(
            id=id,
            start_time=start_time,
            template_url=self.get_utilization_details_for_machine.metadata['url'],
            headers=_headers,
            params=_params,
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)  # type: ignore

        pipeline_response = self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request,
            stream=False,
            **kwargs
        )
        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        deserialized = self._deserialize('UtilizationDetailsDto', pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    get_utilization_details_for_machine.metadata = {'url': "/machineutilization/{id}/utilization"}  # type: ignore

